from unittest import TestCase, TestLoader, TextTestRunner, skipIf
import sys
# deal with absolute imports by adding the appropriate directory to the path
if __name__ == "__main__":
    sys.path.insert(0, "../..")
    from cobra.test import create_test_model
    from cobra import Model, Reaction, Metabolite
    from cobra import solvers
    sys.path.pop(0)  # remove the added directory to the path
else:
    from . import create_test_model
    from .. import Model, Reaction, Metabolite
    from .. import solvers

try:
    import scipy
except:
    scipy = None


class TestCobraSolver(object):
    def setUp(self):
        self.solver = solvers.solver_dict[self.solver_name]
        self.model = create_test_model("textbook")
        self.old_solution = 0.8739215
        self.infeasible_model = Model()
        metabolite_1 = Metabolite("met1")
        reaction_1 = Reaction("rxn1")
        reaction_2 = Reaction("rxn2")
        reaction_1.add_metabolites({metabolite_1: 1})
        reaction_2.add_metabolites({metabolite_1: 1})
        reaction_1.lower_bound = 1
        reaction_2.upper_bound = 2
        self.infeasible_model.add_reactions([reaction_1, reaction_2])

    def test_attributes(self):
        solver = self.solver
        self.assertTrue(hasattr(solver, "create_problem"))
        self.assertTrue(hasattr(solver, "solve_problem"))
        self.assertTrue(hasattr(solver, "get_status"))
        self.assertTrue(hasattr(solver, "get_objective_value"))
        self.assertTrue(hasattr(solver, "format_solution"))
        self.assertTrue(hasattr(solver, "change_variable_bounds"))
        self.assertTrue(hasattr(solver, "change_variable_objective"))
        self.assertTrue(hasattr(solver, "solve"))
        self.assertTrue(hasattr(solver, "set_parameter"))
        # self.assertTrue(hasattr(solver, "update_problem"))

    def test_creation(self):
        solver = self.solver
        solver.create_problem(self.model)

    def test_solve_feasible(self):
        solver = self.solver
        solution = solver.solve(self.model)
        self.assertEqual(solution.status, "optimal")
        self.assertAlmostEqual(self.old_solution,
                               solution.f, places=4)

    def test_solve_minimize(self):
        solver = self.solver
        solution = solver.solve(self.model, objective_sense='minimize')
        self.assertEqual(solution.status, "optimal")
        self.assertAlmostEqual(0, solution.f, places=4)

    def test_low_level_control(self):
        solver = self.solver
        lp = solver.create_problem(self.infeasible_model)
        solver.solve_problem(lp)
        self.assertEqual(solver.get_status(lp), "infeasible")
        # going to make feasible
        solver.change_variable_bounds(lp, 0, -2., 2.)
        solver.change_variable_bounds(lp, 1, -2., 2.)
        solver.solve_problem(lp)
        # should now be feasible, but obj = 0
        self.assertEqual(solver.get_status(lp), "optimal")
        self.assertAlmostEqual(solver.get_objective_value(lp), 0, places=4)
        # should now have obj = 2 (maximize should be the default)
        solver.change_variable_objective(lp, 0, 1.)
        solver.solve_problem(lp)
        self.assertAlmostEqual(solver.get_objective_value(lp), 2, places=4)
        # should now solve with obj = -2
        solver.solve_problem(lp, objective_sense="minimize")
        self.assertAlmostEqual(solver.get_objective_value(lp), -2, places=4)
        # should now have obj = 4
        solver.change_variable_objective(lp, 0, 2.)
        solver.solve_problem(lp, objective_sense="maximize")
        self.assertAlmostEqual(solver.get_objective_value(lp), 4, places=4)
        # make sure the solution looks good still
        solution = solver.format_solution(lp, self.infeasible_model)
        self.assertAlmostEqual(solution.x[0], 2, places=4)
        self.assertAlmostEqual(solution.x[1], -2, places=4)
        self.assertAlmostEqual(solution.x_dict["rxn1"], 2, places=4)
        self.assertAlmostEqual(solution.x_dict["rxn2"], -2, places=4)

    def test_set_objective_sense(self):
        solver = self.solver
        maximize = solver.create_problem(self.model,
                                         objective_sense="maximize")
        minimize = solver.create_problem(self.model,
                                         objective_sense="minimize")
        solver.solve_problem(maximize)
        solver.solve_problem(minimize)
        max_solution = solver.format_solution(maximize, self.model)
        min_solution = solver.format_solution(minimize, self.model)
        self.assertAlmostEqual(0, min_solution.f, places=4)
        self.assertEqual(min_solution.status, "optimal")
        self.assertAlmostEqual(self.old_solution, max_solution.f, places=4)
        self.assertEqual(max_solution.status, "optimal")
        # if we set minimize at creation, can we override it at solve
        solver.solve_problem(minimize, objective_sense="maximize")
        override_minimize = solver.format_solution(minimize, self.model)
        self.assertAlmostEqual(max_solution.f, override_minimize.f, places=4)

    def test_solve_mip(self):
        solver = self.solver
        if not hasattr(solver, "_SUPPORTS_MILP") or not solver._SUPPORTS_MILP:
            self.skipTest("no milp support")
        cobra_model = Model('MILP_implementation_test')
        constraint = Metabolite("constraint")
        constraint._bound = 2.5
        x = Reaction("x")
        x.lower_bound = 0.
        x.objective_coefficient = 1.
        x.add_metabolites({constraint: 2.5})
        y = Reaction("y")
        y.lower_bound = 0.
        y.objective_coefficient = 1.
        y.add_metabolites({constraint: 1.})
        cobra_model.add_reactions([x, y])
        float_sol = solver.solve(cobra_model)
        # add an integer constraint
        y.variable_kind = "integer"
        int_sol = solver.solve(cobra_model)
        self.assertAlmostEqual(float_sol.f, 2.5)
        self.assertAlmostEqual(float_sol.x_dict["y"], 2.5)
        self.assertEqual(int_sol.status, "optimal")
        self.assertAlmostEqual(int_sol.f, 2.2)
        self.assertAlmostEqual(int_sol.x_dict["y"], 2.0)

    def test_solve_infeasible(self):
        solver = self.solver
        solution = solver.solve(self.infeasible_model)
        self.assertEqual(solution.status, "infeasible")

    def test_independent_creation(self):
        solver = self.solver
        feasible_lp = solver.create_problem(self.model)
        infeasible_lp = solver.create_problem(self.infeasible_model)
        solver.solve_problem(feasible_lp)
        solver.solve_problem(infeasible_lp)
        feasible_solution = solver.format_solution(feasible_lp, self.model)
        infeasible_solution = solver.format_solution(infeasible_lp,
                                                     self.infeasible_model)
        self.assertEqual(feasible_solution.status, "optimal")
        self.assertAlmostEqual(self.old_solution,
                               feasible_solution.f, places=4)
        self.assertEqual(infeasible_solution.status, "infeasible")

    def test_change_coefficient(self):
        solver = self.solver
        c = Metabolite("c")
        c._bound = 6
        x = Reaction("x")
        x.lower_bound = 1.
        y = Reaction("y")
        y.lower_bound = 0.
        x.add_metabolites({c: 1})
        z = Reaction("z")
        z.add_metabolites({c: 1})
        z.objective_coefficient = 1
        m = Model("test_model")
        m.add_reactions([x, y, z])
        # change an existing coefficient
        lp = solver.create_problem(m)
        solver.solve_problem(lp)
        sol1 = solver.format_solution(lp, m)
        self.assertEqual(sol1.status, "optimal")
        solver.change_coefficient(lp, 0, 0, 2)
        solver.solve_problem(lp)
        sol2 = solver.format_solution(lp, m)
        self.assertEqual(sol2.status, "optimal")
        self.assertAlmostEqual(sol1.f, 5.0)
        self.assertAlmostEqual(sol2.f, 4.0)
        # change a new coefficient
        z.objective_coefficient = 0.
        y.objective_coefficient = 1.
        lp = solver.create_problem(m)
        solver.change_coefficient(lp, 0, 1, 2)
        solver.solve_problem(lp)
        solution = solver.format_solution(lp, m)
        self.assertEqual(solution.status, "optimal")
        self.assertAlmostEqual(solution.x_dict["y"], 2.5)

    def test_inequality(self):
        # The space enclosed by the constraints is a 2D triangle with
        # vertexes as (3, 0), (1, 2), and (0, 1)
        solver = self.solver
        # c1 encodes y - x > 1 ==> y > x - 1
        # c2 encodes y + x < 3 ==> y < 3 - x
        c1 = Metabolite("c1")
        c2 = Metabolite("c2")
        x = Reaction("x")
        x.lower_bound = 0
        y = Reaction("y")
        y.lower_bound = 0
        x.add_metabolites({c1: -1, c2: 1})
        y.add_metabolites({c1: 1, c2: 1})
        c1._bound = 1
        c1._constraint_sense = "G"
        c2._bound = 3
        c2._constraint_sense = "L"
        m = Model()
        m.add_reactions([x, y])
        # test that optimal values are at the vertices
        m.objective = "x"
        self.assertAlmostEqual(solver.solve(m).f, 1.0)
        self.assertAlmostEqual(solver.solve(m).x_dict["y"], 2.0)
        m.objective = "y"
        self.assertAlmostEqual(solver.solve(m).f, 3.0)
        self.assertAlmostEqual(solver.solve(m, objective_sense="minimize").f,
                               1.0)

    @skipIf(scipy is None, "scipy required for quadratic objectives")
    def test_quadratic(self):
        solver = self.solver
        if not hasattr(solver, "set_quadratic_objective"):
            self.skipTest("no qp support")
        c = Metabolite("c")
        c._bound = 2
        x = Reaction("x")
        x.objective_coefficient = -0.5
        x.lower_bound = 0.
        y = Reaction("y")
        y.objective_coefficient = -0.5
        y.lower_bound = 0.
        x.add_metabolites({c: 1})
        y.add_metabolites({c: 1})
        m = Model()
        m.add_reactions([x, y])
        lp = self.solver.create_problem(m)
        quadratic_obj = scipy.sparse.eye(2) * 2
        solver.set_quadratic_objective(lp, quadratic_obj)
        solver.solve_problem(lp, objective_sense="minimize")
        solution = solver.format_solution(lp, m)
        self.assertEqual(solution.status, "optimal")
        # Respecting linear objectives also makes the objective value 1.
        self.assertAlmostEqual(solution.f, 1.)
        self.assertAlmostEqual(solution.x_dict["y"], 1.)
        self.assertAlmostEqual(solution.x_dict["y"], 1.)
        # When the linear objectives are removed the objective value is 2.
        solver.change_variable_objective(lp, 0, 0.)
        solver.change_variable_objective(lp, 1, 0.)
        solver.solve_problem(lp, objective_sense="minimize")
        solution = solver.format_solution(lp, m)
        self.assertEqual(solution.status, "optimal")
        self.assertAlmostEqual(solution.f, 2.)
        # test quadratic from solve function
        solution = solver.solve(m, quadratic_component=quadratic_obj,
                                objective_sense="minimize")
        self.assertEqual(solution.status, "optimal")
        self.assertAlmostEqual(solution.f, 1.)
        c._bound = 6
        z = Reaction("z")
        x.objective_coefficient = 0.
        y.objective_coefficient = 0.
        z.lower_bound = 0.
        z.add_metabolites({c: 1})
        m.add_reaction(z)
        solution = solver.solve(m, quadratic_component=scipy.sparse.eye(3),
                                objective_sense="minimize")
        # should be 12 not 24 because 1/2 (V^T Q V)
        self.assertEqual(solution.status, "optimal")
        self.assertAlmostEqual(solution.f, 6)
        self.assertAlmostEqual(solution.x_dict["x"], 2, places=6)
        self.assertAlmostEqual(solution.x_dict["y"], 2, places=6)
        self.assertAlmostEqual(solution.x_dict["z"], 2, places=6)

for solver_name in solvers.solver_dict:
    exec('class %sTester(TestCobraSolver, TestCase): None' % solver_name)
    exec('%sTester.solver_name = "%s"' % (solver_name, solver_name))

# make a test suite to run all of the tests
loader = TestLoader()
suite = loader.loadTestsFromModule(sys.modules[__name__])


def test_all():
    TextTestRunner(verbosity=2).run(suite)

if __name__ == "__main__":
    test_all()



import numpy
import pylab

#from ... import solvers
from cobra import solvers
from itertools import combinations


def plot_production_envelope(model, target_id, n_points=20, plot=True,
        solver_name="glpk"):
    """Plot the production envelope for the model given a target

    Parameters
    ----------
    model : cobra model
        The cobra model should already have the uptake rates se
    target_id : str
        The id of the exchange reaction for the target compound
    n_points : int
        The number of points to calculate for the production envolope
    plot : bool, optional
        Whether or not a plot should be made of the production envelope

    Returns
    -------
    growth_rates : :class:`numpy.ndarray`
        An array of growth rates
    production_rates : :class:`numpy.ndarray`
        An array of the corresponding maximum production rate at the
        given growth rate.

    """
    solver = solvers.solver_dict[solver_name]
    target_id = str(target_id)
    target_reaction = model.reactions.get_by_id(target_id)
    original_target_bounds = (target_reaction.lower_bound,
                              target_reaction.upper_bound)
    lp = solver.create_problem(model)
    if solver.solve_problem(lp) != "optimal":
        return ([0], [0])
    solution = solver.format_solution(lp, model)
    max_growth_rate = solution.f
    max_growth_production = solution.x_dict[target_reaction.id]
    #growth_coupled = max_growth_production > 0
    # extract the current objective so it can be changed
    original_objectives = {}
    for reaction in model.reactions:
        if reaction.objective_coefficient != 0:
            original_objectives[reaction] = reaction.objective_coefficient
            reaction.objective_coefficient = 0
    # calculate the maximum possible production rate
    target_reaction.objective_coefficient = 1
    model.optimize(objective_sense="minimize")
    min_production_rate = model.solution.f
    model.optimize(objective_sense="maximize")
    max_production_rate = model.solution.f
    production_rates = numpy.linspace(min_production_rate,
        max_production_rate, n_points)
    # ensure the point of production at maximum growth is included
    production_rates[
        numpy.abs(production_rates - max_growth_production).argmin()] = \
        max_growth_production
    # if the 0 point was overwritten in the last operation
    if production_rates[0] != 0:
        production_rates[1] = production_rates[0]
        production_rates[0] = 0
    growth_rates = production_rates * 0
    # make the objective coefficient what it was before
    target_reaction.objective_coefficient = 0
    for reaction, coefficient in original_objectives.iteritems():
        reaction.objective_coefficient = coefficient
    # calculate the maximum growth rate at each production rate
    for i in range(n_points):
        target_reaction.lower_bound = production_rates[i]
        target_reaction.upper_bound = production_rates[i]
        solver.update_problem(lp, model)
        if solver.solve_problem(lp) == "optimal":
            growth_rates[i] = solver.get_objective_value(lp)
        else:
            growth_rates[i] = 0
    # reset the bounds on the target reaction
    target_reaction.lower_bound = original_target_bounds[0]
    target_reaction.upper_bound = original_target_bounds[1]
    if plot:
        pylab.plot(growth_rates, production_rates)
        pylab.title("Production envelope for %s" % (target_id))
        pylab.xlabel("Growth rate")
        pylab.ylabel("Production rate")
        pylab.xlim(xmin=0)
        pylab.ylim(ymin=0)
    return (growth_rates, production_rates)


def analyze_growth_coupled_num_knockouts(model, knockout_reaction, target_name="EX_etoh_e"):
    None
    

def analyze_growth_coupled_design_subset(model, knockout_reactions, knockout_count, target_name="EX_etoh_e"):
    lp = model.optimize()
    best_score = 0
    best = []
    lb = [None] * k  # store lower bounds when reactions are knocked out
    ub = [None] * k  # store upper bounds when reactions are knocked out
    for subset in combinations(knockout_reactions, knockout_reactions):
        # knockout reactions
        for i, reaction_name in enumerate(subset):
            reaction = model.reactions.get_by_id(str(reaction_name))
            (lb[i], ub[i]) = (reaction.lower_bound, reaction.upper_bound)
            (reaction.lower_bound, reaction.upper_bound) = (0.0, 0.0)
        model.optimize()
        production = model.solution.x_dict[target_name]
        # identical performance
        if abs(production - best_score) < 0.001:
            best.append(subset)
        # better performance
        elif production > best_score:
            best_score = model.solution.x_dict[target_name]
            best = [subset]
        print model.solution.f, model.solution.x_dict[target_name]
        # reset reactions
        for i, reaction_name in enumerate(subset):
            (reaction.lower_bound, reaction.upper_bound) = (lb[i], ub[i])
    return best_score, best

if __name__ == "__main__":
    from cobra.test import ecoli_pickle, create_test_model
    from time import time

    model = create_test_model(ecoli_pickle)
    #from IPython import embed; embed()
    model.reactions.get_by_id("EX_o2_e").lower_bound = 0
    #analyze_strain_design(model, ["ABTA", "ACALD", "ACKr", "ATPS4rpp", "F6PA",
    #          "GLUDy", "LDH_D", "MGSA", "PFL", "TPI"])

    for i in ["ABTA", "ACALD", "ACKr", "ATPS4rpp", "F6PA",
              "GLUDy", "LDH_D", "MGSA", "PFL", "TPI"]:
        model.reactions.get_by_id(i).lower_bound = 0
        model.reactions.get_by_id(i).upper_bound = 0
    start = time()
    plot_production_envelope(model, "EX_etoh_e", solver_name="glpk", n_points=40, plot=True)
    print "ran in %.2f seconds" % (time() - start)
    pylab.show()
    # calculates in approx 1 seconds on 3.4 GHz i7



#cobra.numjy.Matrix.py
#Basic matrix class that is going to be used to mimic numpy.ndarray
#capabilities.
#
#Derived from Simon Galbraith's ncajava Matrix.py file
import java, javax, jarray
from copy import deepcopy
from cern.colt.matrix import DoubleMatrix2D
from cern.colt.matrix.DoubleFactory2D import dense,sparse;
from cern.colt.matrix.impl import DenseDoubleMatrix2D as ndarray
from cern.colt.matrix.impl import SparseDoubleMatrix2D as sdarray
from cern.colt.matrix.linalg import Algebra;
from org.python.core.exceptions import ValueError as PyValueException;
from org.python.core import PyString,PySlice,PySequence,PyList;

class Matrix(javax.swing.table.AbstractTableModel):
    _M = None;   
    _name = 'data matrix'  
    varname = ''
    column_names = []
    row_names = []
           
    def __init__(self,M=None,N=None,v=None,sparse=None):

        """
        M is the number of rows
        N is the number of columns
        v is the default value
        """
        if isinstance(M,DoubleMatrix2D):
            self._M = M.copy();  
        elif (isinstance(M,int) and isinstance(N,int)):
            if sparse:
                F = sparse
            else:
                F = dense;
            if v is None:
                self._M = F.make(M,N,0);
            elif isinstance(v,int): 
                self._M = F.make(M,N,v);
            elif isinstance(v,PyList):
                self._M = F.make(jarray.array(v,'d'),1)
            elif isinstance(v,PyString):
                self._M = F.random(M,N);
            else:
                if sparse:
                    self._M = SparseDoubleMatrix2D(v)
                else:
                    self._M = ndarray(v)
        self.shape = (self._M.rows(),self._M.columns())


    def __copy__(self):
        r = new.instance(self.__class__, self.__dict__.copy() )
        r._M = self._M.copy();
        print "in copy"
        return r
    
    def __sub__(A,B):
        
        [ar,ac]=size(A);        
        C = Matrix(ar,ac,0);
        for i in range(1,ar):
            for j in range(1,ac):
                C[i,j]=A[i,j]-B[i,j]
        return C;
    
    def __mul__(A,B):  
        
        # need to check types and multiple based on them..     
        try:
            F = Algebra();
            C=(F.mult(A._M,B._M));
        except:
            raise PyValueException, "Inner dimension mismatch in matrix multiply.";
            return None;
        return Matrix(C) 
    def __div__(A,B):
        #print size(A)

        try:
            F = Algebra();
            R = F.solve(A._M,B._M);
            return R;
        except (java.lang.IllegalArgumentException) , e :
            # check the error class types according to the matrix class so we can intelligently report the error.
            print e.getMessage();
            return None;        
        
    
    def __repr__(self):
        return self._M.toString();
            
    def __str__(self):
        return self._M.toString();

    def __sz__(self):        
        if isinstance(self,Matrix):
            x=self._M.rows();
            y=self._M.columns();
            return (x,y);
        else:
            raise PyValueException, "Argument must be a matrix.";   
           
    def __setitem__(self,idx,v):

        if v is None:
            print idx
            raise PyValueException, "v is none"
        
        if isinstance(v,float): 
            self._M.set(idx[0],idx[1],v);
            return;
        
        Y=idx[1];
        X=idx[0];
            
        if isinstance(X,PyList):
            X=map(lambda x: x, X)
        elif isinstance(X,PySlice):
            if X.start == None:
               X=range(0,self._M.rows())
        elif isinstance(X,int):
            X=[X];
        
        if isinstance(Y,PyList):
            Y=map(lambda x: x,Y);
        elif isinstance(Y,PySlice):
            if Y.start == None:
               Y=range(0,self._M.cols())        
        elif isinstance(Y,int):
            Y=[Y];

        order=0;
        if len(X)>len(Y):
            order=1;
            
        #print "the order is " , order    
        if order==0:
            y=1
            for q in Y:
                    x=1
                    for z in X: 
         #               print z,q,x,y,v
                        self._M.set(z,q,v[x,y]);
                        x+=1;
                    y+=1;
        else:
            x=1
            for z in X:                    
                    y=1;
                    for q in Y:             
                        self._M.set(z,q,v[x,y]);
                        y+=1;
                    x+=1;
            
                
    def __getslice__(self, i, j):

        if i.start != None:
            x=range(i.start,i.stop);
        else:
            x=range(0,self._M.rows())
        if j.start !=None:    
            y=range(j.start,j.stop)
        else:
            y=range(0,self._M.columns())
        
        return Matrix(self._M.viewSelection(x,y));

         
    def __getitem__(self,idx):
        x=idx[0];
        y=idx[1];
        if x<0 or y<0:
            raise PyValueException, "Index must be positive number";     
          # this will fail on pyslice
        
        if isinstance(x,PySlice):
            if x.start != None:
               x=range(x.start,x.stop);
            else:
               x=range(0,self._M.rows())                   
        elif isinstance(x,int):
            x=x;
            x=[x]; 
        elif isinstance(x,PyList):
            x=map(lambda x: x, x)
       
        if isinstance(y,int):
               y=y;
               y=[y];
        elif isinstance(y,PySlice):
            if y.start != None:
                y=range(y.start,y.stop);
            else:
                y=range(0,self._M.columns())         
        elif isinstance(y,PySlice):
            if y.start !=None:    
                y=range(y.start,y.stop)
            else:
                y=range(0,self._M.columns())
        elif isinstance(y,PyList):
            y=map(lambda x: x, y)

        
        if len(x)<2 and len(y)<2:
            r = self._M.getQuick(x[0],y[0]);
            return float(r)  # this is a specific element
        else:
            return Matrix(self._M.viewSelection(x,y));



from __future__ import absolute_import

import json
from warnings import warn

from .. import Model, Metabolite, Reaction, Gene
from six import iteritems, string_types

# Detect numpy types to replace them.
try:
    from numpy import float_, bool_
except ImportError:
    class float_:
        pass

    class bool_:
        pass

_REQUIRED_REACTION_ATTRIBUTES = {"id", "name", "metabolites", "lower_bound",
                                 "upper_bound", "gene_reaction_rule"}
_OPTIONAL_REACTION_ATTRIBUTES = {
    "objective_coefficient": 0,
    "variable_kind": "continuous",
    "subsystem": "",
    "notes": {},
    "annotation": {},
}

_REQUIRED_METABOLITE_ATTRIBUTES = {"id", "name", "compartment"}
_OPTIONAL_METABOLITE_ATTRIBUTES = {
    "charge": None,
    "formula": None,
    "_bound": 0,
    "_constraint_sense": "E",
    "notes": {},
    "annotation": {},
}

_REQUIRED_GENE_ATTRIBUTES = {"id", "name"}
_OPTIONAL_GENE_ATTRIBUTES = {
    "notes": {},
    "annotation": {},
}

_OPTIONAL_MODEL_ATTRIBUTES = {
    "name": None,
    #  "description": None, should not actually be included
    "compartments": {},
    "notes": {},
    "annotation": {},
}


def _fix_type(value):
    """convert possible types to str, float, and bool"""
    # Because numpy floats can not be pickled to json
    if isinstance(value, string_types):
        return str(value)
    if isinstance(value, float_):
        return float(value)
    if isinstance(value, bool_):
        return bool(value)
    # handle legacy Formula type
    if value.__class__.__name__ == "Formula":
        return str(value)
    if value is None:
        return ''
    return value


def _from_dict(obj):
    """build a model from a dict"""
    if 'reactions' not in obj:
        raise Exception('JSON object has no reactions attribute. Cannot load.')
    model = Model()
    # add metabolites
    new_metabolites = []
    for metabolite in obj['metabolites']:
        new_metabolite = Metabolite()
        for k, v in iteritems(metabolite):
            setattr(new_metabolite, k, v)
        new_metabolites.append(new_metabolite)
    model.add_metabolites(new_metabolites)
    # add genes
    for gene in obj['genes']:
        new_gene = Gene(gene["id"])
        for k, v in iteritems(gene):
            setattr(new_gene, k, _fix_type(v))
        model.genes.append(new_gene)
    # add reactions
    new_reactions = []
    for reaction in obj['reactions']:
        new_reaction = Reaction()
        for k, v in iteritems(reaction):
            if k == 'reversibility' or k == "reaction":
                continue
            elif k == 'metabolites':
                new_reaction.add_metabolites(
                    {model.metabolites.get_by_id(str(met)): coeff
                     for met, coeff in iteritems(v)})
            else:
                setattr(new_reaction, k, _fix_type(v))
        new_reactions.append(new_reaction)
    model.add_reactions(new_reactions)
    for k, v in iteritems(obj):
        if k in {'id', 'name', 'notes', 'compartments', 'annotation'}:
            setattr(model, k, v)
    return model


def _update_optional(cobra_object, new_dict, optional_attribute_dict):
    """update new_dict with optional attributes from cobra_object"""
    for key, default_value in iteritems(optional_attribute_dict):
        value = getattr(cobra_object, key)
        if value is not None and value != default_value:
            new_dict[key] = _fix_type(value)


def _to_dict(model):
    """convert the model to a dict"""
    new_reactions = []
    new_metabolites = []
    new_genes = []
    for reaction in model.reactions:
        new_reaction = {key: _fix_type(getattr(reaction, key))
                        for key in _REQUIRED_REACTION_ATTRIBUTES
                        if key != "metabolites"}
        _update_optional(reaction, new_reaction, _OPTIONAL_REACTION_ATTRIBUTES)
        # set metabolites
        mets = {str(met): coeff for met, coeff
                in iteritems(reaction._metabolites)}
        new_reaction['metabolites'] = mets
        new_reactions.append(new_reaction)
    for metabolite in model.metabolites:
        new_metabolite = {key: _fix_type(getattr(metabolite, key))
                          for key in _REQUIRED_METABOLITE_ATTRIBUTES}
        _update_optional(metabolite, new_metabolite,
                         _OPTIONAL_METABOLITE_ATTRIBUTES)
        new_metabolites.append(new_metabolite)
    for gene in model.genes:
        new_gene = {key: str(getattr(gene, key))
                    for key in _REQUIRED_GENE_ATTRIBUTES}
        _update_optional(gene, new_gene, _OPTIONAL_GENE_ATTRIBUTES)
        new_genes.append(new_gene)
    obj = {'reactions': new_reactions,
           'metabolites': new_metabolites,
           'genes': new_genes,
           'id': model.id,
           }

    _update_optional(model, obj, _OPTIONAL_MODEL_ATTRIBUTES)
    # add in the JSON version
    obj["version"] = 1
    return obj


def to_json(model):
    """Save the cobra model as a json string"""
    return json.dumps(_to_dict(model), allow_nan=False)


def from_json(jsons):
    """Load cobra model from a json string"""
    return _from_dict(json.loads(jsons))


def load_json_model(file_name):
    """Load a cobra model stored as a json file

    file_name : str or file-like object

    """
    # open the file
    should_close = False
    if isinstance(file_name, string_types):
        file_name = open(file_name, 'r')
        should_close = True

    model = _from_dict(json.load(file_name))

    if should_close:
        file_name.close()

    return model


def save_json_model(model, file_name, pretty=False):
    """Save the cobra model as a json file.

    model : :class:`~cobra.core.Model.Model` object

    file_name : str or file-like object

    """
    # open the file
    should_close = False
    if isinstance(file_name, string_types):
        file_name = open(file_name, 'w')
        should_close = True

    if pretty:
        dump_opts = {"indent": 4, "separators": (",", ": "), "sort_keys": True}
    else:
        dump_opts = {}

    json.dump(_to_dict(model), file_name, allow_nan=False, **dump_opts)

    if should_close:
        file_name.close()


json_schema = {
    "$schema": "http://json-schema.org/draft-04/schema#",
    "title": "COBRA",
    "description": "JSON representation of COBRA model",
    "type": "object",
    "properties": {
        "id": {"type": "string"},
        "name": {"type": "string"},
        "description": {"type": "string"},
        "version": {
            "type": "integer",
            "default": 1,
        },

        "reactions": {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "id": {"type": "string"},
                    "name": {"type": "string"},
                    "metabolites": {
                        "type": "object",
                        "patternProperties": {
                            ".*": {"type": "number"},
                        }
                    },
                    "gene_reaction_rule": {"type": "string"},
                    "lower_bound": {"type": "number"},
                    "upper_bound": {"type": "number"},
                    "objective_coefficient": {
                        "type": "number",
                        "default": 0,
                    },
                    "variable_kind": {
                        "type": "string",
                        "pattern": "integer|continuous",
                        "default": "continuous"
                    },
                    "subsystem": {"type": "string"},
                    "notes": {"type": "object"},
                    "annotation": {"type": "object"},
                },
                "required": ["id", "name", "metabolites", "lower_bound",
                             "upper_bound", "gene_reaction_rule"],
                "additionalProperties": False,
            }
        },
        "metabolites": {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "id": {"type": "string"},
                    "name": {"type": "string"},
                    "compartment": {
                        "type": "string",
                        "pattern": "[a-z]{1,2}"
                    },
                    "charge": {"type": "integer"},
                    "formula": {"type": "string"},
                    "_bound": {
                        "type": "number",
                        "default": 0
                    },
                    "_constraint_sense": {
                        "type": "string",
                        "default": "E",
                        "pattern": "E|L|G",
                    },
                    "notes": {"type": "object"},
                    "annotation": {"type": "object"},
                },
                "required": ["id", "name", "compartment"],
                "additionalProperties": False,
            }

        },
        "genes": {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "id": {"type": "string"},
                    "name": {"type": "string"},
                    "notes": {"type": "object"},
                    "annotation": {"type": "object"},
                },
                "required": ["id", "name"],
                "additionalProperties": False,
            }

        },
        "compartments": {
            "type": "object",
            "patternProperties": {
                "[a-z]{1,2}": {"type": "string"}
            }
        },
        "notes": {"type": "object"},
        "annotation": {"type": "object"},
    },
    "required": ["id", "reactions", "metabolites", "genes"],
    "additionalProperties": False,
}



""" Utility functions for sparse matrix module
"""

__all__ = ['upcast','getdtype','isscalarlike','isintlike',
            'isshape','issequence','isdense']

import numjy as np

# keep this list syncronized with sparsetools
#supported_dtypes = ['int8', 'uint8', 'int16', 'uint16', 'int32', 'uint32',
#        'int64', 'uint64', 'float32', 'float64',
#        'complex64', 'complex128']
supported_dtypes = ['int8','uint8','short','ushort','intc','uintc',
        'longlong','ulonglong','single','double','longdouble',
        'csingle','cdouble','clongdouble']
supported_dtypes = [ np.typeDict[x] for x in supported_dtypes]

def upcast(*args):
    """Returns the nearest supported sparse dtype for the
    combination of one or more types.

    upcast(t0, t1, ..., tn) -> T  where T is a supported dtype

    Examples
    --------

    >>> upcast('int32')
    <type 'numpy.int32'>
    >>> upcast('bool')
    <type 'numpy.int8'>
    >>> upcast('int32','float32')
    <type 'numpy.float64'>
    >>> upcast('bool',complex,float)
    <type 'numpy.complex128'>

    """
    sample = np.array([0],dtype=args[0])
    for t in args[1:]:
        sample = sample + np.array([0],dtype=t)

    upcast = sample.dtype

    for t in supported_dtypes:
        if np.can_cast(sample.dtype,t):
            return t

    raise TypeError,'no supported conversion for types: %s' % args


def to_native(A):
    return np.asarray(A,dtype=A.dtype.newbyteorder('native'))


def getdtype(dtype, a=None, default=None):
    """Function used to simplify argument processing.  If 'dtype' is not
    specified (is None), returns a.dtype; otherwise returns a np.dtype
    object created from the specified dtype argument.  If 'dtype' and 'a'
    are both None, construct a data type out of the 'default' parameter.
    Furthermore, 'dtype' must be in 'allowed' set.
    """
    #TODO is this really what we want?
    canCast = True
    if dtype is None:
        try:
            newdtype = a.dtype
        except AttributeError:
            if default is not None:
                newdtype = np.dtype(default)
                canCast = False
            else:
                raise TypeError, "could not interpret data type"
    else:
        newdtype = np.dtype(dtype)

    return newdtype

def isscalarlike(x):
    """Is x either a scalar, an array scalar, or a 0-dim array?"""
    return np.isscalar(x) or (isdense(x) and x.ndim == 0)

def isintlike(x):
    """Is x appropriate as an index into a sparse matrix? Returns True
    if it can be cast safely to a machine int.
    """
    if issequence(x):
        return False
    else:
        try:
            if int(x) == x:
                return True
            else:
                return False
        except TypeError:
            return False

def isshape(x):
    """Is x a valid 2-tuple of dimensions?
    """
    try:
        # Assume it's a tuple of matrix dimensions (M, N)
        (M, N) = x
    except:
        return False
    else:
        if isintlike(M) and isintlike(N):
            if np.rank(M) == 0 and np.rank(N) == 0:
                return True
        return False


def issequence(t):
    #Modded for numjy
    if isinstance(t, (list, tuple)):
        return True
    elif hassattr(t, '._M'):
        return (isinstance(t._M, np.ndarray) and (t.ndim == 1))
    else:
        return False
    

def _isinstance(x, _class):
    ##
    # This makes scipy.sparse.sparse.csc_matrix == __main__.csc_matrix.
    c1 = ('%s' % x.__class__).split( '.' )
    c2 = ('%s' % _class).split( '.' )
    aux = c1[-1] == c2[-1]
    return isinstance(x, _class) or aux

def isdense(x):
    #Modded for numjy
    if hasattr(x,'_M'):
        return _isinstance(x._M, np.ndarray)
    raise Exception('The matrix must be created by numjy')



from os.path import isfile, abspath, dirname, join
from sys import argv, path

# To temporarily modify sys.path
SETUP_DIR = abspath(dirname(__file__))

try:
    from setuptools import setup, find_packages
except ImportError:
    path.insert(0, SETUP_DIR)
    import ez_setup
    path.pop(0)
    ez_setup.use_setuptools()
    from setuptools import setup, find_packages


# for running parallel tests due to a bug in python 2.7.3
# http://bugs.python.org/issue15881#msg170215
try:
    import multiprocessing
except:
    None

# import version to get the version string
path.insert(0, join(SETUP_DIR, "cobra"))
from version import get_version, update_release_version
path.pop(0)
version = get_version(pep440=True)

# If building something for distribution, ensure the VERSION
# file is up to date
if "sdist" in argv or "bdist_wheel" in argv:
    update_release_version()

# cython is optional for building. The c file can be used directly. However,
# for certain functions, the c file must be generated, which requires cython.
try:
    from Cython.Build import cythonize
    from distutils.version import StrictVersion
    import Cython
    try:
        cython_version = StrictVersion(Cython.__version__)
    except ValueError:
        raise ImportError("Cython version not parseable")
    else:
        if cython_version < StrictVersion("0.21"):
            raise ImportError("Cython version too old to use")
except ImportError:
    cythonize = None
    for k in ["sdist", "develop"]:
        if k in argv:
            raise Exception("Cython >= 0.21 required for " + k)

# Begin constructing arguments for building
setup_kwargs = {}

# for building the cglpk solver
try:
    from distutils.extension import Extension
    from distutils.command.build_ext import build_ext
    from os import name
    from platform import system

    class FailBuild(build_ext):
        """allow building of the C extension to fail"""
        def run(self):
            try:
                build_ext.run(self)
            except Exception as e:
                warn(e)

        def build_extension(self, ext):
            try:
                build_ext.build_extension(self, ext)
            except:
                None

    build_args = {}
    setup_kwargs["cmdclass"] = {"build_ext": FailBuild}
    # MAC OS X needs some additional configuration tweaks
    # Build should be run with the python.org python
    # Cython will output C which could generate warnings in clang
    # due to the addition of additional unneeded functions. Because
    # this is a known phenomenon, these warnings are silenced to
    # make other potential warnings which do signal errors stand
    # out.
    if system() == "Darwin":
        build_args["extra_compile_args"] = ["-Wno-unused-function"]

    build_args["libraries"] = ["glpk"]
    # It is possible to statically link libglpk to the built extension. This
    # allows for simplified installation without the need to install libglpk to
    # the system, and is also usueful when installing a particular version of
    # glpk which conflicts with thesystem version. A static libglpk.a can be
    # built by running configure with the export CLFAGS="-fPIC" and copying the
    # file from src/.libs to either the default lib directory or to the build
    # directory. For an example script, see
    # https://gist.github.com/aebrahim/94a2b231d86821f7f225
    include_dirs = []
    library_dirs = []
    if isfile("libglpk.a"):
        library_dirs.append(abspath("."))
    if isfile("glpk.h"):
        include_dirs.append(abspath("."))
    # if the glpk files are not in the current directory attempt to
    # auto-detect their location by finding the location of the glpsol
    # command
    if name == "posix" and len(include_dirs) == 0 and len(library_dirs) == 0:
        from subprocess import check_output
        try:
            glpksol_path = check_output(["which", "glpsol"],
                                        universal_newlines=True).strip()
            glpk_path = abspath(join(dirname(glpksol_path), ".."))
            include_dirs.append(join(glpk_path, "include"))
            library_dirs.append(join(glpk_path, "lib"))
        except Exception as e:
            print('Could not autodetect include and library dirs: ' + str(e))
    if len(include_dirs) > 0:
        build_args["include_dirs"] = include_dirs
    if len(library_dirs) > 0:
        build_args["library_dirs"] = library_dirs
    # use cython if present, otherwise use c file
    if cythonize:
        ext_modules = cythonize([Extension("cobra.solvers.cglpk",
                                           ["cobra/solvers/cglpk.pyx"],
                                           **build_args)],
                                force=True)
    else:
        ext_modules = [Extension("cobra.solvers.cglpk",
                                 ["cobra/solvers/cglpk.c"], **build_args)]
except Exception as e:
    print('Could not build CGLPK: {}'.format(e))
    ext_modules = None

extras = {
    'matlab': ["pymatbridge"],
    'sbml': ["python-libsbml", "lxml"],
    'array': ["numpy>=1.6", "scipy>=11.0"],
    'display': ["matplotlib", "palettable", "pandas"]
}

all_extras = {'Cython>=0.21'}
for extra in extras.values():
    all_extras.update(extra)
extras["all"] = list(all_extras)

# If using bdist_wininst, the installer will not get dependencies like
# a setuptools installation does. Therefore, for the one external dependency,
# which is six.py, we can just download it here and include it in the
# installer.

# The file six.py will need to be manually downloaded and placed in the
# same directory as setup.py.
if "bdist_wininst" in argv:
    setup_kwargs["py_modules"] = ["six"]

try:
    import pypandoc
    readme = pypandoc.convert("README.md", "rst")
    install = pypandoc.convert("INSTALL.md", "rst")
    setup_kwargs["long_description"] = readme + "\n\n" + install
except:
    with open("README.md", "r") as infile:
        setup_kwargs["long_description"] = infile.read()

setup(
    name="cobra",
    version=version,
    packages=find_packages(exclude=['cobra.oven', 'cobra.oven*']),
    setup_requires=[],
    install_requires=["six"],
    tests_require=["jsonschema > 2.5"],
    extras_require=extras,
    ext_modules=ext_modules,

    package_data={
         '': ['test/data/*',
              'VERSION',
              'mlab/matlab_scripts/*m']},

    author="Daniel Robert Hyduke <danielhyduke@gmail.com>, "
    "Ali Ebrahim <aebrahim@ucsd.edu>",
    author_email="aebrahim@ucsd.edu",
    description="COBRApy is a package for constraints-based modeling of "
    "biological networks",
    license="LGPL/GPL v2+",
    keywords="metabolism biology linear programming optimization flux"
    " balance analysis fba",
    url="https://opencobra.github.io/cobrapy",
    test_suite="cobra.test.suite",
    download_url='https://pypi.python.org/pypi/cobra',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU Lesser General Public License v2'
            ' or later (LGPLv2+)',
        'License :: OSI Approved :: GNU General Public License v2'
            ' or later (GPLv2+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Cython',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Bio-Informatics'
    ],
    platforms="GNU/Linux, Mac OS X >= 10.7, Microsoft Windows >= 7",
    **setup_kwargs)



# (c) Simon J Galbraith 2007.
# Daniel Hyduke 2010
import java, jarray
from copy import deepcopy
from cern.colt.list import IntArrayList, DoubleArrayList
from cern.colt.matrix import DoubleFactory2D
from cern.colt.matrix.impl import DenseDoubleMatrix2D
from cern.colt.matrix.impl import SparseDoubleMatrix2D
from cern.colt.matrix.linalg import Algebra, EigenvalueDecomposition, LUDecomposition;
from org.python.core.exceptions import ValueError as PyValueException;
from cern.colt.matrix.doublealgo.Statistic import covariance, correlation;
from cern.jet.math.Functions import abs as cern_abs
from multiarray import ndarray, array
def mean(A):
    """Calculate the mean of Matrix object A

    """
    [r,c]=size(A)
    s=0
    for j in range(1,c):
        s = s+A[r,j]
    return s/c

def sum(A):
    """Calculate the sum of Matrix object A

    """
    return A._M.zSum()


def norm(A,ntype=None):
    F = Algebra();
    if ntype=='fro':
        r=F.normF(A._M);
    elif ntype == 2:
        r=F.norm2(A._M);
    else:
        r=F.norm2(A._M);
    return r;

def rank(A):
    
    if isinstance(A,ndarray):
        F = Algebra();
        r=F.rank(A._M);
        return int(r);
    else:
        raise PyValueException, "Rank function can only be called on matrix objects"

def cond(A):
    F = Algebra();
    return F.cond(A._M);

def size(A):
    return A.__sz__();

def transpose(A):
    F = Algebra();
    if isinstance(A,float):
        return A;
    else:
        return ndarray(F.transpose(A._M));

def inverse(A):
    F = Algebra();
    x=F.inverse(A._M);
    return ndarray(x)


def eig(A):
        # check that _M is square
    try:
        E = EigenvalueDecomposition(A._M);
        U = E.getV();
        eigs = E.getD();
    except PyValueException, e:
        print e;
        raise PyValueException,"Error in eig, check warnings()";  
    return [ndarray(eigs),ndarray(U)];

def solve(A, B):
    F = Algebra();
    if isinstance(A, ndarray) and isinstance(B, float):
            return F.solve(A._M, B);
    elif isinstance(B, ndarray) and  isinstance(A, float):
            return F.solve(A, B._M);
    elif isinstance(A,ndarray) and isinstance(B, ndarray):
            return ndarray(F.solve(A._M, B._M))
    else:
        return A / B

def solve_transpose(A, B):
    F = Algebra();
    if isinstance(A, ndarray) and isinstance(B, float):
            return F.solveTranspose(A._M,B);
    elif isinstance(B, ndarray) and  isinstance(A, float):
            return F.solveTranspose(A, B._M);
    elif isinstance(A, ndarray) and isinstance(B, ndarray):
            return ndarray(F.solveTranspose(A._M, B._M));
    else:
        return A / B

def solve_LR(A, B):
    T = A._M.copy()
    F = LUDecomposition(T);
    if isinstance(A, ndarray) and isinstance(B, float):
            return F.solve(B);
    elif isinstance(A, ndarray) and isinstance(B, ndarray):
        C = F.solve(B._M);
        return ndarray(C)

def cov(A):
    return ndarray(covariance(A._M))

def cor(A):  # handle multidimensional matrix case
    B = cov(A);
    return  ndarray(correlation(B._M))


def abs(A):
    F = cern_abs
    if isinstance(A,float):
        return java.lang.Math.abs(A)
    else:
        X = A._M.assign(F)
        return ndarray(X);

def svd(A):
    X = SingularValueDecomposition(A._M)
    u = X.getU()
    v = X.getV()
    e = X.getS()
    return [ndarray(u), ndarray(e), ndarray(v)]

#TODO:
#Make sure all of the functions below are are defined.  To know
#how they should operate, look at the docstring:
# pydoc numpy.the_function
#
# or in ipython:
# from numpy import *
# ?the_function
#
#
#TODO: These are the java mappings to the type.  If there's any
#difficulties then check the cern.colt data types
int32 = int
int64 = long

def ones(shape, dtype=float, order='C'):
    """Return a new array of given shape and type, filled with ones.
    
     
    See Also
    --------
    zeros
    
    Examples
    --------
    >>> numjy.ones(5)
    array([ 1.,  1.,  1.,  1.,  1.])
    
    >>> numjy.ones((5,), dtype=numjy.int)
    array([1, 1, 1, 1, 1])
    
    >>> numjy.ones((2, 1))
    array([[ 1.],
           [ 1.]])
    
    >>> s = (2,2)
    >>> numjy.ones(s)
    array([[ 1.,  1.],
           [ 1.,  1.]])"""
    return(ndarray(shape[0], shape[1], 1))

def sign(x):
    """ sign(x[, out])
    
    Returns an element-wise indication of the sign of a number.
    
    The `sign` function returns ``-1 if x < 0, 0 if x==0, 1 if x > 0``.
    
    Parameters
    ----------
    x : array_like
      Input values.
    
    Returns
    -------
    y : ndarray
      The sign of `x`.
    
    Examples
    --------
    >>> numjy.sign([-5., 4.5])
    array([-1.,  1.])
    >>> numjy.sign(0)
    0
 
    """
    def sign_int(a_number):
        if a_number < 0:
            return(-1)
        elif a_number == 0:
            return(0)
        else:
            return(1)

    if hasattr(x, '__iter__' ) or isinstance(x, ndarray):
        return_value = array([map(sign_int, list(array(x)._M.toArray()))])
    else:
        #In the case the input is just a number return the sign of the number.
        return_value = sign_int(x)
    return(return_value)       


def vstack(tup):
    """     Stack arrays in sequence vertically (row wise).
    
    Take a sequence of arrays and stack them vertically to make a single
    array. Rebuild arrays divided by `vsplit`.
    Parameters
    ----------
    tup : sequence of ndarrays
        Tuple containing arrays to be stacked. The arrays must have the same
        shape along all but the first axis.
    Returns
    -------
    stacked : ndarray
        The array formed by stacking the given arrays.
    See Also
    --------
    hstack : Stack arrays in sequence horizontally (column wise).
    dstack : Stack arrays in sequence depth wise (along third dimension).
    concatenate : Join a sequence of arrays together.
    vsplit : Split array into a list of multiple sub-arrays vertically.
    
    
    Notes
    -----
    Equivalent to ``np.concatenate(tup, axis=0)``
    
    Examples
    --------
    >>> a = np.array([1, 2, 3])
    >>> b = np.array([2, 3, 4])
    >>> np.vstack((a,b))
    array([[1, 2, 3],
           [2, 3, 4]])
    
    >>> a = np.array([[1], [2], [3]])
    >>> b = np.array([[2], [3], [4]])
    >>> np.vstack((a,b))
    array([[1],
           [2],
           [3],
           [2],
           [3],
           [4]])

           """
    if isinstance(tup[0], sdarray):
        matrix_factory = DoubleFactory2D.sparse
    else:
        #Allow for the case that python lists or tuples are fed to the function
        tup = map(array, tup)
        matrix_factory = DoubleFactory2D.dense

    stacked_matrix = tup[0]._M
    for the_array in tup[1:]:
        stacked_matrix = matrix_factory.appendRows(stacked_matrix, the_array._M)
    return(ndarray(stacked_matrix))


def hstack(tup):
    """
    Stack arrays in sequence horizontally (column wise).

    Take a sequence of arrays and stack them horizontally to make a single array. Rebuild arrays divided by hsplit.

    Parameters:
    tup : sequence of ndarrays
    All arrays must have the same shape along all but the second axis.
    Returns:
    stacked : ndarray
    The array formed by stacking the given arrays.
    See also
    vstack
    Stack arrays in sequence vertically (row wise).
    dstack
    Stack arrays in sequence depth wise (along third axis).
    concatenate
    Join a sequence of arrays together.
    hsplit
    Split array along second axis.
    Notes

    Equivalent to np.concatenate(tup, axis=1)

    Examples

    >>> a = np.array((1,2,3))
    >>> b = np.array((2,3,4))
    >>> np.hstack((a,b))
    array([1, 2, 3, 2, 3, 4])
    >>> a = np.array([[1],[2],[3]])
    >>> b = np.array([[2],[3],[4]])
    >>> np.hstack((a,b))
    array([[1, 2],
           [2, 3],
           [3, 4]])
    """

    if isinstance(tup[0], sdarray):
        matrix_factory = DoubleFactory2D.sparse
    else:
        tup = map(array, tup)
        matrix_factory = DoubleFactory2D.dense

    hstacked_matrix = tup[0]._M
    for the_array in tup[1:]:
        hstacked_matrix = matrix_factory.appendColumns(hstacked_matrix, the_array._M)
    return(ndarray(hstacked_matrix))


def where (condition, x=None, y=None):
    """
    Return a masked array with elements from x or y, depending on condition.

    Returns a masked array, shaped like condition, where the elements
    are from `x` when `condition` is True, and from `y` otherwise.
    If neither `x` nor `y` are given, the function returns a tuple of
    indices where `condition` is True (the result of
    ``condition.nonzero()``).

    Parameters
    ----------
    condition : array_like, bool
        The condition to meet. For each True element, yield the corresponding
        element from `x`, otherwise from `y`.
    x, y : array_like, optional
        Values from which to choose. `x` and `y` need to have the same shape
        as condition, or be broadcast-able to that shape.

    Returns
    -------
    out : MaskedArray or tuple of ndarrays
        The resulting masked array if `x` and `y` were given, otherwise
        the result of ``condition.nonzero()``.

    See Also
    --------
    numpy.where : Equivalent function in the top-level NumPy module.

    Examples
    --------
    >>> x = np.ma.array(np.arange(9.).reshape(3, 3), mask=[[0, 1, 0],
    ...                                                    [1, 0, 1],
    ...                                                    [0, 1, 0]])
    >>> print x
    [[0.0 -- 2.0]
     [-- 4.0 --]
     [6.0 -- 8.0]]
    >>> np.ma.where(x > 5)    # return the indices where x > 5
    (array([2, 2]), array([0, 2]))

    >>> print np.ma.where(x > 5, x, -3.1416)
    [[-3.1416 -- -3.1416]
     [-- -3.1416 --]
     [6.0 -- 8.0]]

    """

##     if x is None and y is None:
##         return filled(condition, 0).nonzero()
##     elif x is None or y is None:
##         raise ValueError, "Either both or neither x and y should be given."
##     # Get the condition ...............
##     fc = filled(condition, 0).astype(MaskType)
##     notfc = np.logical_not(fc)
##     # Get the data ......................................
##     xv = getdata(x)
##     yv = getdata(y)
##     if x is masked:
##         ndtype = yv.dtype
##     elif y is masked:
##         ndtype = xv.dtype
##     else:
##         ndtype = np.find_common_type([xv.dtype, yv.dtype], [])
##     # Construct an empty array and fill it
##     d = np.empty(fc.shape, dtype=ndtype).view(MaskedArray)
##     _data = d._data
##     np.putmask(_data, fc, xv.astype(ndtype))
##     np.putmask(_data, notfc, yv.astype(ndtype))
##     # Create an empty mask and fill it
##     _mask = d._mask = np.zeros(fc.shape, dtype=MaskType)
##     np.putmask(_mask, fc, getmask(x))
##     np.putmask(_mask, notfc, getmask(y))
##     _mask |= getmaskarray(condition)
##     if not _mask.any():
##         d._mask = nomask
##     return d








## def where(condition  , tup = 'blank', arg3 = 'blank'):








##     def conditioncheck(x):
##         if x == 'True':
##             return(1)
##         else:
##             return(0)

##     if isinstance(condition, bool):
##         truth_matrix = zeros(z.columns(), z.rows())
##         for i in xrange(truth_matrix.rows):
##             for j in xrange(truth_matrix.columns):
##                 truth_matrix.set(i,j, conditioncheck(z.get(i,j)))
##     else:
##         truth_matrix = zeros((len(condition), len(condition[0])))._M
##         for i in xrange(truth_matrix.rows()):
##             for j in xrange(truth_matrix.columns()):
##                 truth_matrix.set(i,j, conditioncheck(condition[i][j]))

##     if arg3 == 'blank':
##         if tup == 'blank':
##                 row_list = IntArrayList()
##                 column_list = IntArrayList()
##                 coordinate_list = DoubleArrayList()
##                 truth_matrix.getNonZeros(row_list, column_list, coordinate_list)
##                 return( (array(row_list), array(column_list)) )
##         else:
##             value_matrix = list(tup)
##             for i, x in enumerate(value_matrix):
##                 if isinstance(x, int):
##                     matfac = DoubleFactory2D.dense
##                     matrix = matfac.make(truth_matrix.rows(), truth_matrix.columns(), x)
##                     value_matrix[i] = matrix
##                 else:
##                     value_matrix[i] = x._M
##             return_array = zeros((truth_matrix.rows(), truth_matrix.columns()))._M
##             for i in xrange(truth_matrix.rows()):
##                 for j in xrange(truth_matrix.columns()):
##                     if truth_matrix.get(i,j) == 1:
##                         return_array.set(i,j, value_matrix[0].get(i,j))
##                     else:
##                         return_array.set(i,j, value_matrix[1].get(i,j))
##     else:
##         value_matrix = [ tup , arg3 ]
##         for i, x in enumerate(value_matrix):
##             if isinstance(x, int):
##                 matfac = DoubleFactory2D.dense
##                 matrix = matfac.make(truth_matrix.rows(), truth_matrix.columns(), x)
##                 value_matrix[i] = matrix
##             else:
##                 value_matrix[i] = x
##         return_array = zeros((truth_matrix.rows(), truth_matrix.columns()))._M
##         for i in xrange(truth_matrix.rows()):
##             for j in xrange(truth_matrix.columns()):
##                 if truth_matrix.get(i,j) == 1:
##                     print(type(value_matrix[0]))
##                     return_array.set(i,j, value_matrix[0].get(i,j))
##                 else:
##                     return_array.set(i,j, value_matrix[1].get(i,j))


##     return( return_array )

def nonzero(array):
    """Return the indices of the elements that are non-zero.
    Returns a tuple of arrays, one for each dimension of a, containing the indices of the non-zero elements
    in that dimension. The corresponding non-zero values can be obtained with:
    Parameters:	
    a : array_like
    Input array.
    Returns:	
    tuple_of_arrays : tuple
    Indices of elements that are non-zero.

    >>> x = np.eye(3)

    >>> x
    array([[ 1.,  0.,  0.],
    [ 0.,  1.,  0.],
    [ 0.,  0.,  1.]])

    >>> np.nonzero(x)
    (array([0, 1, 2]), array([0, 1, 2]))

    >>> x[np.nonzero(x)]
    array([ 1.,  1.,  1.])

    >>> np.transpose(np.nonzero(x))
    array([[0, 0],
       [1, 1],
       [2, 2]])"""
    
    rowList = IntArrayList()
    columnList = IntArrayList()
    coordinateList = DoubleArrayList()
    array._M.getNonZeros(rowList, columnList, coordinateList)
#TODO update array function to deal with Int....
    return(array(rowList), array(columnList))

def repeat(array, repeat_tup, axis = 2):
    """
    Repeat elements of an array.

    Parameters:
    a : array_like
    Input array.
    repeats : {int, array of ints}
    The number of repetitions for each element. repeats is broadcasted to fit the shape of the given axis.
    axis : int, optional
    The axis along which to repeat values. By default, use the flattened input array, and return a flat output array.
    Returns:
    repeated_array : ndarray
    Output array which has the same shape as a, except along the given axis.
    See also
    tile
    Tile an array.
    Examples

    >>> x = np.array([[1,2],[3,4]])
    >>> np.repeat(x, 2)
    array([1, 1, 2, 2, 3, 3, 4, 4])
    >>> np.repeat(x, 3, axis=1)
    array([[1, 1, 1, 2, 2, 2],
           [3, 3, 3, 4, 4, 4]])
    >>> np.repeat(x, [1, 2], axis=0)
    array([[1, 2],
           [3, 4],
           [3, 4]])
        """
    repeat = list(repeat_tup)
    #TODO: Make sure this can handle sparse matrices as well.  See the vstack function for ideas.
    matrix_factory = DoubleFactory2D.dense
    if axis == 1:
        repeat_matrix = matrix_factory.repeat(array._M.viewPart(0,0,array._M.rows(),1),1,repeat[0])
        for i in range( array._M.columns())[1:]:
            repeated_slice =  matrix_factory.repeat(array._M.viewPart(0, i, array._M.rows(), 1) , 1, repeat[i])
            repeat_matrix = matrix_factory.appendColumns(repeat_matrix, repeated_slice )
        return(repeat_matrix)
    elif axis == 0:
        repeat_matrix = matrix_factory.repeat(array._M.viewPart(0,0,1,array._M.columns()),repeat[0],1)
        for i in range(array._M.columns())[1:]:   
            repeat_matrix = matrix_factory.appendRows(repeat_matrix,
                                                      matrix_factory.repeat(array._M.viewPart(i,0,1,array._M.columns()),
                                                                            repeat[i], 1))
            return(repeat_matrix)
    else:
        pass





#One
#From scipy sparse.lil_matrix, sparse.hstack, sparse.vstack

def matrix():
    #This is not pressing.  I updated my cobra modules to use array instead of
    #matrix so this can be dealt with later.
    pass



    





from __future__ import absolute_import, print_function

import mosek

from six.moves import zip
from six import iteritems, string_types

env = mosek.Env()

# make sure the mosek environment works
test = env.Task(0, 0)
test.optimize()
del test


solver_name = "mosek"
__mosek_version__ = ".".join(str(i) for i in mosek.getversion())
_SUPPORTS_MILP = True

status_dict = {
    mosek.solsta.dual_infeas_cer: 'infeasible',
    mosek.solsta.prim_infeas_cer: 'infeasible',
    mosek.solsta.optimal: 'optimal',
    mosek.solsta.integer_optimal: 'optimal'}

param_enums = {
    "bi_clean_optimizer": mosek.optimizertype,
    "cache_license": mosek.onoffkey,
    "check_convexity": mosek.checkconvexitytype,
    "compress_statfile": mosek.onoffkey,
    "feasrepair_optimize": mosek.feasrepairtype,
    "infeas_generic_names": mosek.onoffkey,
    "infeas_prefer_primal": mosek.onoffkey,
    "infeas_report_auto": mosek.onoffkey,
    "license_allow_overuse": mosek.onoffkey,
    "license_wait": mosek.onoffkey,
    "mio_branch_dir": mosek.branchdir,
    "mio_branch_priorities_use": mosek.onoffkey,
    "mio_construct_sol": mosek.onoffkey,
    "mio_cont_sol": mosek.miocontsoltype,
    "mio_use_multithreaded_optimizer": mosek.onoffkey,
    "presolve_lindep_use": mosek.onoffkey,
    "presolve_use": mosek.presolvemode,
    "sim_basis_factor_use": mosek.onoffkey,
    "sim_degen": mosek.simdegen
}

param_aliases = {
    "tolerance_feasibility": "basis_tol_x",
    "time_limit": "optimizer_max_time",
    "mip_gap_abs": "mio_tol_abs_gap",
    "mip_gap": "mio_tol_rel_gap",
    "tolerance_integer": "mio_tol_abs_relax_int"
    }


def _verbose_printer(text):
    print(text)


def create_problem(cobra_model, objective_sense="maximize",
                   **solver_parameters):
    n_rxns = len(cobra_model.reactions)
    n_mets = len(cobra_model.metabolites)
    rxn_indexes = range(n_rxns)
    met_indexes = range(n_mets)
    # create lp object and set default parameters
    lp = env.Task(0, 0)
    set_objective_sense(lp, objective_sense)
    lp.putdouparam(mosek.dparam.basis_tol_x, 1e-9)
    lp.putdouparam(mosek.dparam.basis_tol_s, 1e-9)
    lp.putdouparam(mosek.dparam.basis_rel_tol_s, 0.)
    lp.putdouparam(mosek.dparam.simplex_abs_tol_piv, 1e-12)
    lp.putdouparam(mosek.dparam.presolve_tol_aij, 1e-15)
    lp.putdouparam(mosek.dparam.presolve_tol_abs_lindep, 0.)
    lp.putdouparam(mosek.dparam.presolve_tol_s, 0.)
    lp.putdouparam(mosek.dparam.presolve_tol_x, 1e-10)
    lp.putintparam(mosek.iparam.concurrent_priority_intpnt, 0)
    lp.putintparam(mosek.iparam.concurrent_num_optimizers, 1)
    # add reactions/variables
    lp.appendvars(len(cobra_model.reactions))
    lp.putvarboundlist(
        rxn_indexes,
        (mosek.boundkey.ra,) * n_rxns,
        [float(i.lower_bound) for i in cobra_model.reactions],
        [float(i.upper_bound) for i in cobra_model.reactions],
    )
    lp.putclist(
        rxn_indexes,
        [float(i.objective_coefficient) for i in cobra_model.reactions])
    integer_variables = [i for i, r in enumerate(cobra_model.reactions)
                         if r.variable_kind == "integer"]
    lp.putvartypelist(
        integer_variables,
        (mosek.variabletype.type_int,) * len(integer_variables))

    # add metabolites/constraints
    c_sense_dict = {"E": mosek.boundkey.fx, "L": mosek.boundkey.up,
                    "G": mosek.boundkey.lo}
    c_senses = [c_sense_dict[met._constraint_sense]
                for met in cobra_model.metabolites]
    bounds = cobra_model.metabolites.list_attr("_bound")
    lp.appendcons(len(cobra_model.metabolites))
    lp.putconboundlist(met_indexes, c_senses, bounds, bounds)

    # add in the S matrix
    for i, reaction in enumerate(cobra_model.reactions):
        for metabolite, stoichiometry in iteritems(reaction._metabolites):
            lp.putaij(cobra_model.metabolites.index(metabolite),
                      i, stoichiometry)
    # set user-supplied parameters
    for key, value in iteritems(solver_parameters):
        set_parameter(lp, key, value)
    return lp


def set_objective_sense(lp, objective_sense):
    if objective_sense == "maximize":
        lp.putobjsense(mosek.objsense.maximize)
    elif objective_sense == "minimize":
        lp.putobjsense(mosek.objsense.minimize)
    else:
        raise ValueError("unknown objective sense '%s'" % objective_sense)


def set_parameter(lp, parameter_name, parameter_value):
    parameter_name = param_aliases.get(parameter_name, parameter_name)
    if parameter_name == "verbose":
        if parameter_value:
            for streamtype in mosek.streamtype.values:
                lp.set_Stream(streamtype, _verbose_printer)
        else:
            for streamtype in mosek.streamtype.values:
                lp.set_Stream(streamtype, lambda x: None)
    elif parameter_name == "objective_sense":
        set_objective_sense(lp, parameter_value)
    elif parameter_name == "quadratic_component":
        if parameter_value is not None:
            set_quadratic_objective(lp, parameter_value)
    elif hasattr(mosek.dparam, parameter_name):
        lp.putdouparam(getattr(mosek.dparam, parameter_name), parameter_value)
    # Integer parameter section
    # Many of these are enumerations. Therefore the value should be converted
    # to the appropriate enum one.
    elif isinstance(parameter_value, string_types) and \
            parameter_name in param_enums:
        lp.putintparam(getattr(mosek.iparam, parameter_name),
                       getattr(param_enums[parameter_name], parameter_value))
    elif isinstance(parameter_value, bool) and parameter_name in param_enums:
        if parameter_value:
            set_parameter(lp, parameter_name, "on")
        else:
            set_parameter(lp, parameter_name, "off")
    elif hasattr(mosek.iparam, parameter_name):
        lp.putintparam(getattr(mosek.iparam, parameter_name), parameter_value)
    else:
        raise ValueError("unknown parameter '%s'" % parameter_name)


def solve_problem(lp, **solver_parameters):
    for key, value in iteritems(solver_parameters):
        set_parameter(lp, key, value)
    lp.optimize()
    return get_status(lp)


def solve(cobra_model, **solver_parameters):
    lp = create_problem(cobra_model, **solver_parameters)
    solve_problem(lp)
    return format_solution(lp, cobra_model)


def _get_soltype(lp):
    """get the solution type

    This is bas for LP and itg for MIP and QP

    """
    if lp.getnumintvar() > 0:
        return mosek.soltype.itg
    if lp.getnumqobjnz() > 0:
        return mosek.soltype.itr
    return mosek.soltype.bas


def get_status(lp):
    mosek_status = lp.getsolsta(_get_soltype(lp))
    return status_dict.get(mosek_status, str(mosek_status))


def get_objective_value(lp):
    return lp.getprimalobj(_get_soltype(lp))


def format_solution(lp, cobra_model):
    soltype = _get_soltype(lp)
    mosek_status = lp.getsolsta(soltype)
    status = status_dict.get(mosek_status, str(mosek_status))
    if status != "optimal":
        return cobra_model.solution.__class__(None, status=status)
    solution = cobra_model.solution.__class__(get_objective_value(lp))
    solution.status = status
    x = [0] * len(cobra_model.reactions)
    lp.getxx(soltype, x)
    solution.x = x
    solution.x_dict = {rxn.id: value for rxn, value
                       in zip(cobra_model.reactions, x)}
    if soltype == mosek.soltype.bas:
        y = [0] * len(cobra_model.metabolites)
        lp.gety(mosek.soltype.bas, y)
        solution.y = y
        solution.y_dict = {met.id: value for met, value
                           in zip(cobra_model.metabolites, y)}
    return solution


def change_variable_objective(lp, index, value):
    lp.putcj(index, value)


def change_variable_bounds(lp, index, lower_bound, upper_bound):
    lp.putvarbound(index, mosek.boundkey.ra, lower_bound, upper_bound)


def change_coefficient(lp, met_index, rxn_index, value):
    lp.putaij(met_index, rxn_index, value)


def set_quadratic_objective(lp, quadratic_objective):
    if not hasattr(quadratic_objective, 'todok'):
        raise Exception('quadratic component must be a sparse matrix')
    row_indexes = []
    col_indexes = []
    values = []
    for (index_0, index_1), value in iteritems(quadratic_objective.todok()):
        # specify lower triangular only
        if index_0 >= index_1:
            row_indexes.append(index_0)
            col_indexes.append(index_1)
            values.append(value)
    lp.putqobj(row_indexes, col_indexes, values)



from numpy import ndarray
class ResultsArray(ndarray):
    """A primitive wrapper to allow accessing numpy.ndarrays via
    named rows and columns.  The ResultsArray.row_names and
    column_names must be assigned after the object is created.

    The names will not carry over for any operations.

    TODO: Finish the implementation

    """
    def __init__(self, shape, row_names=None, column_names=None):
        ndarray.__init__(shape)
        if row_names:
            self.row_names = row_names
        else:
            self.row_names = range(shape[0])
        if column_names:
            self.column_names = column_names
        else:
            column_names = range(shape[1])
    def get(self, row_name=None, column_name=None):
        if row_name:
            the_row = self.row_names.index(row_name)
        if column_name:
            the_column = self.column_names.index(column_name)
        if row_name and column_name:
            return self[the_row, the_column]
        if not row_name:
            return self[:, the_column]
        if not column_name:
                return self[the_row, :]



import csv
import re
import copy
from os.path import join, abspath, split

import cobra

# the default file locations
kegg_directory = join(split(abspath(__file__))[0], "kegg_files")
keggdictpath_default = join(kegg_directory, "kegg_dict.csv")
reactionlst_default = join(kegg_directory, "reaction.lst")
blacklistpath_default = join(kegg_directory, "kegg_blacklist.csv")


def _intify(string):
    """returns integer representation of the str
    If str is a single letter, it will return 1"""
    if string.isdigit():
        return int(string)
    # if the expression contains n, the default value is 2
    n = 2
    if string == "2n":
        return 2 * n
    try:
        return eval(string)
    except:
        raise ValueError(string)


def _parse_split_array(str_array):
    """takes in an array of strings, each of which is either
    - a compound OR
    - a number followed by a compound
    returns [array_of_metabolites, corresponding_coefficient]"""
    metabolites = []
    coefficients = []
    for string in str_array:
        string = string.strip()
        if string[0].isupper():  # starts with an uppercase letter
            # there is no number associated, so it should be 1
            metabolites.append(string)
            coefficients.append(1)
        else:
            the_coefficient, the_metabolite = string.split()
            metabolites.append(the_metabolite)
            coefficients.append(_intify(the_coefficient))
    return [metabolites, coefficients]


def import_kegg_reactions(compartment="c", reactionlstpath=None,
                        keggdictpath=None, blacklistpath=None):
    """reads in kegg reactions from the three given files
    compartment: the compartment to which each reaction will be added

    If no file is specified for any of these, a default file will be used:
    reactionlstpath: path to path of kegg reactions
    the format should be
    reactionid: Met1 + 2 Met2 <=> Met3 + 2 Met4

    keggdictpath: The path to a csv file translating between kegg and cobra
    metabolite ID's, where the first column contains the kegg ID, and the
    second contains cobra id

    blacklistpath: path to a file listing the blacklisted reactions, with
    one per line

    returns: cobra model with all of the included reactions
    """

    if reactionlstpath is None:
        reactionlstpath = reactionlst_default
    if keggdictpath is None:
        keggdictpath = keggdictpath_default
    if blacklistpath is None:
        blacklistpath = blacklistpath_default

    # read in kegg dictionary to translate between kegg and cobra id's
    keggdictfile = open(keggdictpath, "r")
    keggdictcsv = csv.reader(keggdictfile)
    keggdict = {}
    for line in keggdictcsv:
        keggdict[line[1]] = line[0]
    keggdictfile.close()
    # read in the kegg blacklist
    keggblacklistfile = open(blacklistpath, "r")
    keggblacklistcsv = csv.reader(keggblacklistfile)
    keggblacklist = []
    for line in keggblacklistcsv:
        keggblacklist.append(line[0])
    keggblacklistfile.close()

    # parse the file of kegg reactions
    keggfile = open(reactionlstpath, "r")
    # regular expressions to split strings
    colon_sep = re.compile(":").split
    arrow_sep = re.compile("<=>").split
    plus_sep = re.compile(" \+ ").split
    keggreactions = []
    cobra_reactions = []
    used_metabolites = {}
    for line in keggfile:
        [id, reactionstr] = colon_sep(line, maxsplit=1)
        # remove whitespace
        id = id.strip()
        # if the id is in the blacklist, no need to proceed
        if id in keggblacklist:
            continue
        # split into reactants and products
        reactants_str, products_str = arrow_sep(reactionstr, maxsplit=1)
        # break up reactant and product strings into arrays of
        # metabolites and coefficients
        reactant_metabolites, reactant_coefficients = \
            _parse_split_array(plus_sep(reactants_str))
        product_metabolites, product_coefficients = \
            _parse_split_array(plus_sep(products_str))
        # reactant coefficients all need to be multiplied by -1
        for i, coeff in enumerate(reactant_coefficients):
            reactant_coefficients[i] = coeff * -1
        # make one array for all compoenents
        kegg_metabolites = reactant_metabolites
        coefficients = reactant_coefficients
        kegg_metabolites.extend(product_metabolites)
        coefficients.extend(product_coefficients)
        # translate the metabolites from kegg to cobra
        metabolites = []
        try:
            for the_kegg_metabolite in kegg_metabolites:
                metabolites.append(keggdict[the_kegg_metabolite])
        # if one of the metabolites is not found, skip to the next line
        except KeyError:
            continue

        # make a Kegg reaction
        reaction = cobra.Reaction(id)
        metabolite_dict = {}  # dict of {metabolite : coefficient}
        for i, the_metabolite in enumerate(metabolites):
            metabolite_id = the_metabolite + "_" + compartment
            # if the metabolite already exists
            if metabolite_id in used_metabolites:
                used_metabolites[metabolite_id] = coefficients[i]
            else:
                # use a new metabolite
                new_metabolite = cobra.Metabolite(metabolite_id)
                used_metabolites[metabolite_id] = new_metabolite
                metabolite_dict[cobra.Metabolite(metabolite_id)] = \
                    coefficients[i]
        reaction.add_metabolites(metabolite_dict)
        reaction.notes["temporary_gapfilling_type"] = "Universal"
        # because the model will be converted to irreversible
        reaction.lower_bound = -1 * reaction.upper_bound
        cobra_reactions.append(reaction)
    keggfile.close()
    # add all of the reactions to a cobra model
    Universal = cobra.Model("Kegg_Universal_Reactions")
    Universal.add_reactions(cobra_reactions)
    return Universal
if __name__ == "__main__":
    from time import time
    start_time = time()
    test_import = import_kegg_reactions()
    duration = time() - start_time
    print "imported %d reactions in %.2f sec" % \
        (len(test_import.reactions), duration)



from __future__ import print_function

from ..core import Model, Reaction, Metabolite
from ..solvers import get_solver_name
from ..manipulation import modify


class SUXModelMILP(Model):
    """Model with additional Universal and Exchange reactions.
    Adds corresponding dummy reactions and dummy metabolites for each added
    reaction which are used to impose MILP constraints to minimize the
    total number of added reactions. See the figure for more
    information on the structure of the matrix.
    """
    def __init__(self, model, Universal=None, threshold=.05,
                 penalties=None, dm_rxns=True, ex_rxns=False):
        Model.__init__(self, "")
        # store parameters
        self.threshold = threshold
        if penalties is None:
            self.penalties = {"Universal": 1, "Exchange": 100, "Demand": 1}
        else:
            self.penalties = penalties
        # want to only operate on a copy of Universal so as not to mess up
        # is this necessary?
        if Universal is None:
            Universal = Model("Universal_Reactions")
        else:
            Universal = Universal.copy()

        modify.convert_to_irreversible(Universal)

        for rxn in Universal.reactions:
            rxn.notes["gapfilling_type"] = "Universal"

        # SUX += Exchange (when exchange generator has been written)
        # For now, adding exchange reactions to Universal - could add to a new
        # model called exchange and allow their addition or not....
        if ex_rxns:
            for m in model.metabolites:
                rxn = Reaction('SMILEY_EX_' + m.id)
                rxn.lower_bound = 0
                rxn.upper_bound = 1000
                rxn.add_metabolites({m: 1.0})
                rxn.notes["gapfilling_type"] = "Exchange"
                Universal.add_reaction(rxn)

        if dm_rxns:
            # ADD DEMAND REACTIONS FOR ALL METABOLITES TO UNIVERSAL MODEL
            for m in model.metabolites:
                rxn = Reaction('SMILEY_DM_' + m.id)
                rxn.lower_bound = 0
                rxn.upper_bound = 1000
                rxn.add_metabolites({m: -1.0})
                rxn.notes["gapfilling_type"] = "Demand"
                Universal.add_reaction(rxn)

        Model.add_reactions(self, model.copy().reactions)
        Model.add_reactions(self, Universal.reactions)

        # all reactions with an index < len(model.reactions) were original
        self.original_reactions = self.reactions[:len(model.reactions)]
        self.added_reactions = self.reactions[len(model.reactions):]

        # Add MILP indicator reactions
        indicators = []
        for reaction in self.added_reactions:
            dummy_metabolite = Metabolite("dummy_met_" + reaction.id)
            dummy_metabolite._constraint_sense = "L"
            reaction.add_metabolites({dummy_metabolite: 1})
            indicator_reaction = Reaction("indicator_" + reaction.id)
            indicator_reaction.add_metabolites(
                {dummy_metabolite: -1 * reaction.upper_bound})
            indicator_reaction.lower_bound = 0
            indicator_reaction.upper_bound = 1
            indicator_reaction.variable_kind = "integer"
            indicator_reaction.objective_coefficient = \
                self.penalties[reaction.notes["gapfilling_type"]]
            indicators.append(indicator_reaction)
        Model.add_reactions(self, indicators)

        # original reaction objectives need to be set to lower bounds
        self._update_objectives()

    def _update_objectives(self, added=True):
        """Update the metabolite which encodes the objective function
        with the objective coefficients for the reaction, and impose
        penalties for added reactions.
        """
        for reaction in self.original_reactions:
            if reaction.objective_coefficient > 0:
                reaction.lower_bound = max(
                    reaction.lower_bound,
                    reaction.objective_coefficient * self.threshold)
            reaction.objective_coefficient = 0

    def add_reactions(self, reactions):
        Model.add_reactions(self, reactions)
        self.original_reactions.extend(reactions)
        self._update_objectives()

    def solve(self, solver=None, iterations=1, debug=False, time_limit=100,
              **solver_parameters):
        """solve the MILP problem"""
        if solver is None:
            solver = get_solver_name(mip=True)
        used_reactions = [None] * iterations
        numeric_error_cutoff = 0.0001
        self._update_objectives()
        for i in range(iterations):
            used_reactions[i] = []
            self.optimize(objective_sense="minimize",
                          solver=solver, **solver_parameters)
            if debug:
                print("Iteration %d: Status is %s" % (i, self.solution.status))
            for reaction in self.added_reactions:
                # The dummy reaction should have a flux of either 0 or 1.
                # If it is 1 (nonzero), then the reaction was used in
                # the solution.
                ind = self.reactions.get_by_id("indicator_" + reaction.id)
                if ind.x > numeric_error_cutoff:
                    used_reactions[i].append(reaction)
                    ind.objective_coefficient += \
                        self.penalties[reaction.notes["gapfilling_type"]]
                    if debug:
                        print('    ', reaction, reaction.objective_coefficient)

        return used_reactions


def growMatch(model, Universal, dm_rxns=False, ex_rxns=False,
              penalties=None, **solver_parameters):
    """runs growMatch"""
    SUX = SUXModelMILP(model, Universal, dm_rxns=dm_rxns, ex_rxns=ex_rxns,
                       penalties=penalties)
    return SUX.solve(**solver_parameters)


def SMILEY(model, metabolite_id, Universal,
           dm_rxns=False, ex_rxns=False, panalties=None, **solver_parameters):
    """
    runs the SMILEY algorithm to determine which gaps should be
    filled in order for the model to create the metabolite with the
    given metabolite_id.

    This function is good for running the algorithm once. For more fine-
    grained control, create a SUXModelMILP object, add a demand reaction
    for the given metabolite_id, and call the solve function on the
    SUXModelMILP object.
    """
    SUX = SUXModelMILP(model, Universal, dm_rxns=dm_rxns, ex_rxns=ex_rxns,
                       penalties=penalties)
    # change the objective to be the metabolite
    for reaction in SUX.original_reactions:
        reaction.objective_coefficient = 0
    demand_name = "SMILEY_DM_" + metabolite_id
    if demand_name not in SUX.reactions:
        demand_reaction = Reaction(demand_name)
        demand_reaction.add_metabolites(
            {SUX.metabolites.get_by_id(metabolite_id): -1})
        SUX.add_reaction(demand_reaction)
    else:
        demand_reaction = SUX.reactions.get_by_id(demand_name)
    demand_reaction.lower_bound = SUX.threshold
    return SUX.solve(**solver_parameters)
